/*
 * ouvej, determines what, when and where the next lesson will be
 * Copyright (C) 2003 Olivier Mehani <shtrom@ssji.net, olmehani@etu.utc.fr>
 * 
 * $Id: ouvej.h,v 1.26 2004/03/07 16:23:32 shtrom Exp $
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef _OUVEJ_H
#define _OUVEJ_H

#include <stdio.h>
#include <time.h>

#ifdef DEBUG 
# define DPRINT(fmt, args...)	fprintf(stderr,"[%s:%d] %s: "fmt,__FILE__,__LINE__,__FUNCTION__ , ##args)
# define DDO(what)		what
#else
# define DPRINT(fmt, args...)	while(0) /* debug statement */
# define DDO(what)		DPRINT(what)
#endif

#define ERROR(fmt, args...)	fprintf(stderr,"Error: "fmt , ##args)

#define MAXLINESIZE	128
#define RL_EOF		-1
#define RL_OK		0
#define RL_AGAIN	1
#define RL_USELESS	2

#define L_NAME_LEN	32
#define L_PLACE_LEN	8

#define NAME	"ouvej"
#define MAJOR	0
#define MINOR	1
#define REV	0

/*
 * Violently stolen from pot's skin_data.h and console.c
 */
#define BLACK	0
#define RED	1
#define GREEN	2
#define YELLOW	3
#define BLUE	4
#define PINK	5
#define CYAN	6
#define WHITE	7

#define FOREGROUND	30
#define BACKGROUND	40

#define BOLD		1
#define UNDERLINED	4
#define BLINK		5
#define REVERSE		7

/*
 * to be used in printf like functions
 */
#define ANSI_STRING	"\033[%dm"

#define OPT_DUMP_BTREE	1<<0

struct hour {
	int hour;
	int minutes;
};
typedef struct hour hour_t;

enum week_day {
	D_MONDAY,
	D_TUESDAY,
	D_WENESDAY,
	D_THURSDAY,
	D_FRIDAY,
	D_SATURDAY,
	D_SUNDAY,
	D_ERROR
};
typedef enum week_day week_day_t;

enum lesson_type {
	L_C,	/* cours */
	L_D,	/* td */
	L_T,	/* tp */
	L_ERROR
};
typedef enum lesson_type lesson_type_t;

enum week_sequence {
	W_BOTH,
	W_A,
	W_B,
	W_N,
	W_ERROR
};
typedef enum week_sequence week_sequence_t;

struct lesson {
	char name[L_NAME_LEN];
	lesson_type_t type;	
	int group;	
	week_day_t day;
	week_sequence_t week_seq;
	hour_t beginning;
	hour_t end;
	char place[L_PLACE_LEN];
};
typedef struct lesson lesson_t;

#define L_T_ITEMS	8

struct ll {
	lesson_t lesson;
	struct ll *next;
};
typedef struct ll ll_t;

struct bt {
	lesson_t lesson;
	struct bt *earlier;
	struct bt *later;
};
typedef struct bt bt_t;

/*
 * in btree.c
 */
bt_t *btree_make(ll_t *list);
bt_t *btree_make_equilibrated(int size, lesson_t lessons[size]);
void btree_add(bt_t **btree, lesson_t l);
bt_t *btree_new_node(lesson_t l);
int btree_write(bt_t *btree, char *filename);
bt_t *btree_read(char *filename);
lesson_t btree_get_smallest_and_remove(bt_t **btree);
lesson_t btree_get_smallest(bt_t *btree);
void btree_show(bt_t *btree);
lesson_t btree_find_next(bt_t *bt, lesson_t l);

/*
 * in cache.c
 */
int cache_check(char *homedir);
FILE *_cache_ropen(char *cachename, int iter);
FILE *cache_ropen(char *cachename);
FILE *cache_wopen(char *cachename);
int cache_regen(char *ttpath, char *cachepath);
char *get_homedir(char **envp);
char *timetable_path(char *homedir);
char *cache_path(char *homedir);

/*
 * in lesson.c
 */
ll_t *gen_list(char *filename);
int read_line(FILE *utt, char *buffer);
lesson_t gen_lesson(char *b);
char *next_block(char *b);
char *trim(char *p);
int only_spaces(char *b);

lesson_t *lesson_read(char *filename, int *num);

int lesson_set_name(lesson_t *l, char *name);
int lesson_set_type(lesson_t *l, char *type);
int lesson_set_group(lesson_t *l, char *group);
int lesson_set_day(lesson_t *l, char *day);
int lesson_set_seq(lesson_t *l, char *seq);
int lesson_set_beg(lesson_t *l, char *beg);
int lesson_set_end(lesson_t *l, char *end);
int lesson_set_place(lesson_t *l, char *place);

int __lesson_set_seq(lesson_t *l, week_sequence_t seq);

lesson_type_t type_from_string(char *s);
int group_from_string(char *s);
week_day_t day_from_string(char *s);
week_sequence_t seq_from_string(char *s);
hour_t time_from_string(char *s);

char *lesson_get_name(lesson_t *l);
lesson_type_t lesson_get_type(lesson_t *l);
int lesson_get_group(lesson_t *l);
week_day_t lesson_get_day(lesson_t *l);
week_sequence_t lesson_get_seq(lesson_t *l);
hour_t lesson_get_beg(lesson_t *l);
hour_t lesson_get_end(lesson_t *l);
char *lesson_get_place(lesson_t *l);

int lesson_match_week_seq(lesson_t model, lesson_t l);
int lesson_compare_beg(lesson_t l1, lesson_t l2);
int lesson_compare_end(lesson_t l1, lesson_t l2);
int hour_compare(hour_t t1, hour_t t2);

void lesson_show(lesson_t l,FILE *out);
lesson_t lesson_from_time(time_t t);

/*
 * in user.c
 */
void user_version();
void user_usage(char *callname);
void user_clean_ansi();
void user_set_ansi(int fg, int bg, int style);
void user_show_current(lesson_t right_now);
void user_show_lesson(lesson_t l, lesson_t right_now);
char *user_day_name(week_day_t d);
char *user_lesson_type(lesson_type_t lt);
char *user_week_sequence(week_sequence_t seq);
char *user_time(hour_t t);
char *user_min2hours(int minutes);

/*
 * in week-*.c
 */
week_sequence_t get_this_week_sequence(); 

#endif /* _OUVEJ_H */
