/*
 * ttd, a time table displayer 
 * Copyright (C) 2004 Olivier Mehani <shtrom@ssji.net>
 * 
 * $Id: ttd.c,v 1.3 2004/03/07 19:40:31 shtrom Exp $
 * main functions
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <string.h>
#include <errno.h>
#include "ouvej.h"
#include "ttd.h"

static lesson_t *lesson_read(char *filename, int *num);

int main(int argc, char **argv, char **envp)
{
	GtkWidget *mainwin;
	lesson_t *ll;
	char *homedir;
	int num;
	
	gtk_init(&argc,&argv);

	homedir=get_homedir(envp);

	cache_check(homedir);

	cache_check(homedir);

	ll=lesson_read(cache_path(homedir),&num);

	mainwin=ttd_gtk_mainwin(num,ll);

	gtk_main();

	return 0;
}

lesson_t *lesson_read(char *filename, int *num)
{
/*
 * ReadBufferIncrement every realloc will allocate memory space for RBI lessons
 * 
 * A "normal" semester may contain about 20 differents lessons
 */
#define RBI	20
	lesson_t *llist=NULL;
	int remains=0;
	FILE *in; 
	int fm;

	*num=0;

	DPRINT("reading lessons from %s\n",filename);

	in=fopen(filename,"r");
	if(!in) {
		ERROR("can't open cache file %s for reading\n",filename);
		exit(1);
	}

	fread(&fm,1,sizeof(fm),in);

	while(!feof(in)) {
		if(!remains) {
			DPRINT("no space left, reallocating\n");
			llist=realloc(llist,(*num+RBI)*sizeof(lesson_t));
			if(!llist) {
				ERROR("can't allocate memory (%s)\n", strerror(errno));
				exit(1);
			}
			remains=RBI;
		}
		
		if(fread(&llist[*num],1,sizeof(lesson_t),in)==sizeof(lesson_t)) {
			/*
			 * Hey ! we successfully read a lesson_t !
			 * 
			 * Champagne !
			 */
			(*num)++;
			remains--;
		} else {
			DPRINT("couldn't read a complete lesson_t\n");
		}
	}
	fclose(in);

	DPRINT("EOF, read %d lesson_t_n, now adjusting memory\n",*num);
	llist=realloc(llist,*num*sizeof(lesson_t));
	if(!llist) {
		ERROR("can't allocate memory (%s)\n", strerror(errno));
		exit(1);
	}

	return llist;
#undef RBI
}

