/*
 * ouvej, determines what, when and where the next lesson will be
 * Copyright (C) 2003 Olivier Mehani <shtrom@ssji.net, olmehani@etu.utc.fr>
 * 
 * $Id: user.c,v 1.13 2004/03/07 18:47:59 shtrom Exp $
 * pretty user output functions 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include <string.h>
#include "ouvej.h"

void user_version()
{
	printf("ouvej %d.%d.%d (compiled %s, %s)\n",MAJOR,MINOR,REV,__DATE__,__TIME__);
	printf("Copyright (c) 2003 Olivier Mehani <shtrom@ssji.net, olmehani@etu.utc.fr>\n\n");
	printf("This is free software; see the source for copying conditions.  There is NO\nwarranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");
}

void user_usage(char *callname)
{
	printf("Usage: %s [OPTIONS]\n\n",callname);
	printf("Options:\n");
	printf("\t-b\t\tDump the binary tree to screen\n");
	printf("\t-h\t\tThis message\n");
	printf("\t-v\t\tShow version information\n");
}

void user_clean_ansi()
{
	printf(ANSI_STRING,0);
}

void user_set_ansi(int fg, int bg, int style)
{
	user_clean_ansi();
	if(style>=0) printf(ANSI_STRING,style);
	if(fg>=0) printf(ANSI_STRING,FOREGROUND+fg);
	if(bg>=0) printf(ANSI_STRING,BACKGROUND+bg);
}

void user_show_current(lesson_t right_now)
{
	printf("Nous sommes %s (%s), il est %s\n",user_day_name(lesson_get_day(&right_now)),user_week_sequence(lesson_get_seq(&right_now)),user_time(lesson_get_end(&right_now)));
}

void user_show_lesson(lesson_t l, lesson_t right_now)
{
	int offset;

	if(!l.name[0]) {
		/* if the day is friday or after */
		if(l.day>3) {
			printf("Week End !\n");
		} else {
			printf("Cong !\n");
		}
	} else if (l.day>right_now.day) {
		printf("C'est fini pour aujourd'hui\n");
		printf("Prochain rendez-vous %s a %s, %s de %s en %s\n",
			user_day_name(l.day),
			user_time(l.beginning),
		 	user_lesson_type(l.type), 
			l.name,
			l.place);
	} else {
		printf("A %s, %s de %s en %s\n",
			user_time(l.beginning),
		 	user_lesson_type(l.type), 
			l.name,
			l.place);
		offset=hour_compare(l.beginning,right_now.beginning);
		if(offset<0) {
			printf(ANSI_STRING"A la bourre de %s !\n",FOREGROUND+RED,user_min2hours(offset));
		} else if(offset>0) {
			printf("Dans %s...\n",user_min2hours(offset));
		} else {
			printf("Juste maintenant\n");
		}
	}
}

char *user_day_name(week_day_t d)
{
	char *dayname[]={"Lundi", "Mardi", "Mercredi", "Jeudi", "Vendredi", "Samedi","Dimanche", "ERREUR"};
	
	if(d<D_MONDAY || d>D_SUNDAY)
		return dayname[7];
	
	return dayname[d];
}

char *user_lesson_type(lesson_type_t lt)
{
	char *ltype[]={"cours","TD","TP","ERREUR"};

	if(lt<L_C || lt>L_T)
		return ltype[3];
	
	return ltype[lt];
}

char *user_week_sequence(week_sequence_t seq)
{
	char *wseq[]={"Tout le temps","Semaine A","Semaine B","glande !","ERREUR"};
	
	if(seq<W_BOTH || seq>W_N)
		return wseq[4];
	
	return wseq[seq];
}

/*
 * WARNING: the returned string will only remain valid until the next call to
 * user_time()
 */
char *user_time(hour_t t)
{
	static char buffer[6];
	snprintf(buffer,6,"%02d:%02d",t.hour,t.minutes);
	return buffer;
}

/*
 * WARNING: the returned string will only remain valid until the next call to
 * user_min2hours()
 */
char *user_min2hours(int minutes)
{
#define BUFFER_SIZE	32	
	static char buffer[BUFFER_SIZE];
	char tmp[BUFFER_SIZE];
	int hours;

	*buffer=0;

	if(minutes<0)
		minutes=-minutes;

	hours=minutes/60;
	minutes=minutes%60;

	if(hours) {
		snprintf(tmp, BUFFER_SIZE-1, "%d heure", hours);

		if(hours>1)
			strcat(tmp,"s");
		
		strcat(buffer,tmp);
	}

	if(hours && minutes)
		strcat(buffer," et ");

	if(minutes) { 
		snprintf(tmp, BUFFER_SIZE-1, "%d minute", minutes);
	
		if(minutes>1)
			strcat(tmp,"s");
	
		strcat(buffer,tmp);
	}
	
	return buffer;
#undef BUFFER_SIZE
}
